﻿// Originally developed by Andreas Guenther.

namespace TabsStudioAutoOpener
{
    public class AutoOpener : TabsStudioExt.ITabsStudioAddin, TabsStudioExt.IConfigurable
    {
        public void OnConnection(TabsStudioExt.ITabsStudioEngine engine_, EnvDTE80.DTE2 dte_)
        {
            engine = engine_;
            dte = dte_;

            storage = new OptionsStorage();
            options = storage.Load();

            timer = new System.Windows.Threading.DispatcherTimer();
            timer.Interval = System.TimeSpan.FromMilliseconds(1);
            timer.Tick += new System.EventHandler(TimerTick);

            engine.TabCreated += OnTabEvent;

            solutionEventsSink = new SolutionEventsSink();
            System.IServiceProvider serviceProvider = new Microsoft.VisualStudio.Shell.ServiceProvider(
                    dte as Microsoft.VisualStudio.OLE.Interop.IServiceProvider);
            vsSolution = serviceProvider.GetService(typeof(Microsoft.VisualStudio.Shell.Interop.SVsSolution)) as
                Microsoft.VisualStudio.Shell.Interop.IVsSolution;
            vsSolution.AdviseSolutionEvents(solutionEventsSink, out sinkCookie);
        }

        public void OnDisconnection()
        {
            vsSolution.UnadviseSolutionEvents(sinkCookie);
            engine.TabCreated -= OnTabEvent;
            timer.Stop();
        }

        public void Configure(System.Windows.Window parent)
        {
            OptionsDialog dialog = new OptionsDialog(options);
            dialog.Owner = parent;
            dialog.WindowStartupLocation = System.Windows.WindowStartupLocation.CenterOwner;
            if (dialog.ShowDialog().GetValueOrDefault())
                storage.Save(options);
        }

        private void OnTabEvent(object sender, TabsStudioExt.TabEventArgs e)
        {
            if (solutionEventsSink.IsSolutionOpening())
                return;

            foreach (string file in engine.GetCorrespondingFiles(e.Tab.TabItem))
                if (IsFileAllowed(file))
                    dte.ExecuteCommand("File.OpenFile", '"' + file + '"');

            for (int i = 0; i < e.Tab.TabExtensions.Count; i++)
            {
                if (e.Tab.TabExtensions[i].IsExtensionActive)
                {
                    if (e.Tab.TabExtensions[i].Window != dte.ActiveWindow)
                    {
                        windowToActivate = e.Tab.TabExtensions[i].Window;
                        timer.Start();
                    }
                    break;
                }
            }
        }

        private void TimerTick(object sender, System.EventArgs e)
        {
            timer.Stop();
            if (windowToActivate != null)
            {
                windowToActivate.Activate();
                windowToActivate = null;
            }
        }

        private bool IsFileAllowed(string file)
        {
            string extension = GetExtension(System.IO.Path.GetFileName(file));
            if (extension.Length <= 1)
                return false;
            
            foreach (string allowedExtension in options.allowedExtensions.Split(','))
                if (string.Compare(extension, "." + allowedExtension, true) == 0)
                    return true;

            return false;
        }

        private string GetExtension(string fileName)
        {
            int firstDot = fileName.IndexOf('.');
            if (firstDot < 0)
                return "";
            return fileName.Substring(firstDot);
        }

        private TabsStudioExt.ITabsStudioEngine engine;
        private EnvDTE80.DTE2 dte;
        private OptionsStorage storage;
        private Options options;
        private Microsoft.VisualStudio.Shell.Interop.IVsSolution vsSolution;
        private SolutionEventsSink solutionEventsSink;
        private uint sinkCookie;
        private System.Windows.Threading.DispatcherTimer timer;
        private EnvDTE.Window windowToActivate;
    }
}
